package cz.autocont.iszr.demo.e100;

import java.net.MalformedURLException;
import java.net.URL;
import java.text.MessageFormat;
import java.util.UUID;

import cz.isvs.iszr.schemas.iszrasyncodpovedzfronty.v1.IszrAsyncOdpovedZFrontyResponseType;
import cz.isvs.iszr.schemas.iszrasyncodpovedzfronty.v1.IszrAsyncOdpovedZFrontyType;
import cz.autocont.iszr.demo.AsyncDemoCall;
import cz.autocont.iszr.demo.BaseDemoCall;
import cz.autocont.iszr.demo.DemoException;
import cz.autocont.iszr.demo.e99.DemoCallE99;
import cz.isvs.iszr.services.iszrasyncodpovedzfronty.v1.IszrAsyncOdpovedZFrontyPortType;
import cz.isvs.iszr.services.iszrasyncodpovedzfronty.v1.IszrAsyncOdpovedZFrontyService;

import javax.xml.namespace.QName;
import jakarta.xml.ws.BindingProvider;
import jakarta.xml.ws.Response;
import jakarta.xml.ws.WebServiceClient;

/**
 * Demostrace asynchronniho volani webove sluzby E100 - IszrAsyncOdpovedZFronty.
 * Volani je naimplentovano jako aktivni cekani. Druhou moznosti je implementovat cekani pomoci call-back
 * metody. Tento pristup je implementovany v {@link AsyncDemoCall}.
 * <div style="font:italic">Projekt ISZR - (c) AutoCont CZ a.s. All rights reserved</div>
 * 
 * @author Vladimir Pach
 * @since 0.0.1
 */
public class DemoCallE100 extends BaseDemoCall<IszrAsyncOdpovedZFrontyType, IszrAsyncOdpovedZFrontyResponseType>
{
    /**
     * Vytvori data, ktera budou odeslana do webove sluzby.
     * 
     * @return Odesilana data.
     */
    @Override
    protected IszrAsyncOdpovedZFrontyType createRequestData()
    {
        IszrAsyncOdpovedZFrontyType result = new IszrAsyncOdpovedZFrontyType();

        result.setZadostInfo(createZadostInfo());
        result.setKodAsyncSluzby(getConfig().getProperty("E100.kodAsyncSluzby"));
        result.setZadost(createIszrAsyncOdpovedZFrontyZadost());

        return result;
    }

    /**
     * Zavola webovou sluzbu E100.
     * 
     * @param requestData Data, ktera budou odeslana na webovou sluzbu.
     * @return Data, ktera vratila webova sluzba.
     * @throws DemoException Pokud pri volani sluzby dojde k chybe.
     */
    @Override
    protected IszrAsyncOdpovedZFrontyResponseType callService(IszrAsyncOdpovedZFrontyType requestData) throws DemoException
    {
        // get service port
        IszrAsyncOdpovedZFrontyPortType port = createServicePort();

        System.out.println("Pro komunikaci budou pouzity tyto hodnoty:");
        System.out.println("Agenda=" + requestData.getZadostInfo().getAgenda());
        System.out.println("Role=" + requestData.getZadostInfo().getAgendovaRole());
        System.out.println("Ais=" + requestData.getZadostInfo().getAis());
        System.out.println("Ovm=" + requestData.getZadostInfo().getOvm());
        String target = (String) ((BindingProvider) port).getRequestContext().get(BindingProvider.ENDPOINT_ADDRESS_PROPERTY);
        System.out.println("Probiha asynchronni volani webove sluzby " + target);

        // call service
        Response<IszrAsyncOdpovedZFrontyResponseType> handler = port.iszrAsyncOdpovedZFrontyAsync(requestData.getKodAsyncSluzby(),
                requestData.getZadostInfo(), requestData.getZadost());

        try
        {
            // wait for result
            IszrAsyncOdpovedZFrontyResponseType response = handler.get();
            System.out.println("Dokonceno asynchronni volani webove sluzby " + target);
            // return result
            return response;
        }
        catch (Exception e)
        {
            throw new DemoException("Error by callign web serivce E100.", e);
        }
    }

    /**
     * Provede zpracovani dat, ktera vatila webova sluzba.
     * 
     * @param responseData Data, ktera vratila wbova sluzba.
     */
    @Override
    protected void processResponseData(IszrAsyncOdpovedZFrontyResponseType responseData)
    {
        System.out.println("Pridelene IszrZadostId:" + responseData.getOdpovedInfo().getIszrZadostId());
        System.out.println("Vysledek:" + responseData.getOdpovedInfo().getStatus().getVysledekKod());
    }

    /**
     * Vytvori datovou cast pozadavku.
     * V configuraci pod klice "E100.iszrZadostId" je ulozeno ID ISZR zadosti, ktere bude vlozeno do pozadavku a odeslano do webove sluzby.
     * Tento klic naplnilo spusteno demostrace {@link DemoCallE99}. Pokud klic neni nastaveny (je null), pak bude nahodne vygenerovane nove ID.
     * 
     * @return Data pozadavku
     */
    protected IszrAsyncOdpovedZFrontyType.Zadost createIszrAsyncOdpovedZFrontyZadost()
    {
        IszrAsyncOdpovedZFrontyType.Zadost result = new IszrAsyncOdpovedZFrontyType.Zadost();
        result.setIszrAsyncOdpovedZFrontyData(new IszrAsyncOdpovedZFrontyType.Zadost.IszrAsyncOdpovedZFrontyData());

        String iszrId = getConfig().getProperty("E100.iszrZadostId");
        iszrId = iszrId != null ? iszrId : UUID.randomUUID().toString();

        result.getIszrAsyncOdpovedZFrontyData().setIszrZadostId(iszrId);
        return result;
    }

    /**
     * Vytvori proxy port na webovou sluzbu.
     * 
     * @return Proxy port
     * @throws DemoException Pokud pri vytvareni portu dojde k chybe.
     */
    private IszrAsyncOdpovedZFrontyPortType createServicePort() throws DemoException
    {
        WebServiceClient webServiceClient = IszrAsyncOdpovedZFrontyService.class.getAnnotation(WebServiceClient.class);
        final String TARGET_ENDPOINT_ADDRESS = MessageFormat.format(getConfig().getProperty("iszr.location"), "E100");
        final String TARGET_SERVICE_NAMESPACE = webServiceClient.targetNamespace();
        final String TARGET_SERVICE_NAME = webServiceClient.name();
        final String TARGET_WSDL_LOCATION = MessageFormat.format(getConfig().getProperty("wsdl.location"), "IszrAsyncOdpovedZFronty.wsdl");

        // get service
        IszrAsyncOdpovedZFrontyService service;
        try
        {
            service = new IszrAsyncOdpovedZFrontyService(new URL(TARGET_WSDL_LOCATION), new QName(TARGET_SERVICE_NAMESPACE, TARGET_SERVICE_NAME));
        }
        catch (MalformedURLException e)
        {
            throw new DemoException("Wrong target address: " + TARGET_ENDPOINT_ADDRESS, e);
        }
        IszrAsyncOdpovedZFrontyPortType port = service.getIszrAsyncOdpovedZFrontyPortService();

        // setup target adress and SOAP loging
        setUpServicePort(port, "E100", TARGET_ENDPOINT_ADDRESS);

        return port;
    }
}
