package cz.autocont.iszr.demo.e278;

import cz.autocont.iszr.demo.BaseDemoCall;
import cz.autocont.iszr.demo.DemoException;
import cz.isvs.iszr.schemas.iszrabstract.v1.AutorizaceType;
import cz.isvs.iszr.schemas.iszrabstract.v1.ZadostInfoIszrType;
import cz.isvs.iszr.schemas.iszrrobctipodleudaju2.v1.RobCtiPodleUdaju2ResponseType;
import cz.isvs.iszr.schemas.iszrrobctipodleudaju2.v1.RobCtiPodleUdaju2Type;
import cz.isvs.iszr.services.iszrrobctipodleudaju2.v1.IszrRobCtiPodleUdaju2PortType;
import cz.isvs.iszr.services.iszrrobctipodleudaju2.v1.IszrRobCtiPodleUdaju2Service;
import cz.isvs.rob.schemas.robdotazydata.v1.RobCtiPodleUdaju2DataType;
import cz.isvs.rob.schemas.robtypy.v1.AdresaPobytuType;

import javax.xml.namespace.QName;
import jakarta.xml.ws.BindingProvider;
import jakarta.xml.ws.WebServiceClient;
import java.net.MalformedURLException;
import java.net.URL;
import java.text.MessageFormat;

/**
 * Demostrace synchronniho volani webove sluzby E278 - IszrRobCtiPodleUdaju2.
 * <div style="font:italic">Projekt ISZR - (c) AutoCont CZ a.s. All rights reserved</div>
 * 
 * @author Vladimir Pach
 * @since 0.0.1
 */
public class DemoCallE278 extends BaseDemoCall<RobCtiPodleUdaju2Type, RobCtiPodleUdaju2ResponseType>
{
    /**
     * Vytvori data, ktera budou odeslana do webove sluzby.
     * 
     * @return Odesilana data.
     */
    @Override
    protected RobCtiPodleUdaju2Type createRequestData()
    {
        RobCtiPodleUdaju2Type result = new RobCtiPodleUdaju2Type();

        result.setZadostInfo(createZadostInfo());
        result.setAutorizaceInfo(createAutorizaceInfo());
        result.setZadost(createRobCtiPodleUdajuZadost());

        return result;
    }

    /**
     * Zavola webovou sluzbu E278.
     * 
     * @param requestData Data, ktera budou odeslana na webovou sluzbu.
     * @return Data, ktera vratila webova sluzba.
     * @throws DemoException Pokud pri volani sluzby dojde k chybe.
     */
    @Override
    protected RobCtiPodleUdaju2ResponseType callService(RobCtiPodleUdaju2Type requestData) throws DemoException
    {
        // get service port
        IszrRobCtiPodleUdaju2PortType port = createServicePort();

        System.out.println("Pro komunikaci budou pouzity tyto hodnoty:");
        System.out.println("Agenda=" + requestData.getZadostInfo().getAgenda());
        System.out.println("Role=" + requestData.getZadostInfo().getAgendovaRole());
        System.out.println("Ais=" + requestData.getZadostInfo().getAis());
        System.out.println("Ovm=" + requestData.getZadostInfo().getOvm());
        String target = (String) ((BindingProvider) port).getRequestContext().get(BindingProvider.ENDPOINT_ADDRESS_PROPERTY);
        System.out.println("Probiha synchronni volani webove sluzby " + target);

        // call service
        return port.iszrRobCtiPodleUdaju2(requestData);
    }

    /**
     * Provede zpracovani dat, ktera vatila webova sluzba.
     * 
     * @param responseData Data, ktera vratila wbova sluzba.
     */
    @Override
    protected void processResponseData(RobCtiPodleUdaju2ResponseType responseData)
    {
        System.out.println("Pridelene IszrZadostId:" + responseData.getOdpovedInfo().getIszrZadostId());
        System.out.println("Vysledek:" + responseData.getOdpovedInfo().getStatus().getVysledekKod());
    }

    /**
     * Spolecna hlavicka vsech externich zadosti nebo dotazu (webovych sluzeb). <br/>
     * Vytvori zakladn strukturu zadosti pro sluzbu ISZR. {@link ZadostInfoIszrType} obsahuje informace
     * o volajici strane a danem pozadavku. Na zaklade techto dat ISZR rozhodne, o tom, jestli ma volajici strana pravo vykonat volanou sluzbu.
     * 
     * @return Vyplnene informace o volajici strane.
     */
    @Override
    protected ZadostInfoIszrType createZadostInfo()
    {
        ZadostInfoIszrType result = super.createZadostInfo();

        result.setSubjekt(getConfig().getProperty("E278.subjekt"));
        result.setUzivatel(getConfig().getProperty("E278.uzivatel"));
        result.setDuvodUcel(getConfig().getProperty("E278.duvodUcel"));

        return result;
    }

    /**
     * Seznam autorizacnich udaju.
     * 
     * @return seznam hodnot.
     */
    protected AutorizaceType createAutorizaceInfo()
    {
        AutorizaceType autorizaceType = new AutorizaceType();
        String[] values = getConfig().getProperty("E278.autorizace.udaje", "").split("[\\s]*\\|[\\s]*");
        for (String item : values)
        {
            autorizaceType.getSeznamUdaju().add(item);
        }
        return autorizaceType;
    }

    /**
     * Vytvori datovou cast pozadavku.
     * 
     * @return Data pozadavku
     */
    protected RobCtiPodleUdaju2Type.Zadost createRobCtiPodleUdajuZadost()
    {
        RobCtiPodleUdaju2Type.Zadost result = new RobCtiPodleUdaju2Type.Zadost();
        result.setRobCtiPodleUdaju2Data(new RobCtiPodleUdaju2DataType());
        result.getRobCtiPodleUdaju2Data().setPrijmeni(getConfig().getProperty("E278.prijmeni"));
        result.getRobCtiPodleUdaju2Data().setJmeno(getConfig().getProperty("E278.jmeno"));

        AdresaPobytuType adresaPobytuType = new AdresaPobytuType();
        adresaPobytuType.setValue(Integer.parseInt(getConfig().getProperty("E278.adresaPobytu")));
        adresaPobytuType.setAdresaUradu(Boolean.parseBoolean(getConfig().getProperty("E278.adresaUradu")));
        result.getRobCtiPodleUdaju2Data().setAdresaPobytu(adresaPobytuType);

        return result;
    }

    /**
     * Vytvori proxy port na webovou sluzbu.
     * 
     * @return Proxy port
     * @throws DemoException Pokud pri vytvareni portu dojde k chybe.
     */
    private IszrRobCtiPodleUdaju2PortType createServicePort() throws DemoException
    {
        WebServiceClient webServiceClient = IszrRobCtiPodleUdaju2Service.class.getAnnotation(WebServiceClient.class);
        final String TARGET_ENDPOINT_ADDRESS = MessageFormat.format(getConfig().getProperty("iszr.location"), "E278");
        final String TARGET_SERVICE_NAMESPACE = webServiceClient.targetNamespace();
        final String TARGET_SERVICE_NAME = webServiceClient.name();
        final String TARGET_WSDL_LOCATION = MessageFormat.format(getConfig().getProperty("wsdl.location"), "IszrRobCtiPodleUdaju2.wsdl");

        // get service
        IszrRobCtiPodleUdaju2Service service;
        try
        {
            service = new IszrRobCtiPodleUdaju2Service(new URL(TARGET_WSDL_LOCATION), new QName(TARGET_SERVICE_NAMESPACE, TARGET_SERVICE_NAME));
        }
        catch (MalformedURLException e)
        {
            throw new DemoException("Wrong target address: " + TARGET_ENDPOINT_ADDRESS, e);
        }
        IszrRobCtiPodleUdaju2PortType port = service.getIszrRobCtiPodleUdaju2PortService();

        // setup target adress and SOAP loging
        setUpServicePort(port, "E278", TARGET_ENDPOINT_ADDRESS);
        return port;
    }
}
