package cz.autocont.iszr.demo.trust;

import java.io.File;

/**
 * Factory pro vytvoreni instance printeru pro tisk obsahu uloziste klicu a certifikatu.
 * <div style="font:italic">Projekt ISZR - (c) AutoCont CZ a.s. All rights reserved</div>
 * 
 * @author Vladimir Pach
 * @since 0.0.1
 */
public class KeyStorePrinterFactory
{
    /**
     * Vrati factory pro vytvoreni daneho printeru. Aktualne je podporovane uloziste typu jks.
     * 
     * @param storeType Typ uloziste. Pokud je null, pak je vracena factory pro uloziste typu jks.
     * @return Instance factory. Nikdy nevrati null
     * @throws UnsupportedOperationException Pokud storeType obsahuje nepodporovany typ uloziste.
     */
    public static KeyStorePrinterFactory getInstance(String storeType)
    {

        if (storeType == null || "jks".equals(storeType))
        {
            return new KeyStorePrinterFactory();
        }
        else
        {
            throw new UnsupportedOperationException("Unsupportetd key store type");
        }
    }

    /**
     * Vytvori novou instanci printeru pro dany typ uloziste.
     * 
     * @param store Soubor uloziste.
     * @param password Heslo k ulozisti. Pokud uloziste heslo nema, pak obsahuje null.
     * @return Printer pro dany typ uloziste.
     */
    public KeyStorePrinter createPrinter(File store, String password)
    {
        return new KeyStorePrinter(store, password);
    }
}
